from . import constants
from .generated_client import PocketOptionClient
from .models import (
    Asset,
    AuthorizationData,
    ChangeAssetRequest,
    Command,
    CopySignalRequest,
    Deal,
    DealAction,
    IsDemo,
    OpenDealRequest,
    OpenPendingDealRequest,
    OpenPendingDealRequestOpenType,
    SuccessUpdateBalanceEvent,
    UpdateCloseValueItem,
    UpdateCloseValueListTypeAdapter,
    UpdateHistoryFastEvent,
)

__all__ = (
    "Asset",
    "AuthorizationData",
    "ChangeAssetRequest",
    "Command",
    "CopySignalRequest",
    "Deal",
    "IsDemo",
    "OpenDealRequest",
    "OpenPendingDealRequest",
    "OpenPendingDealRequestOpenType",
    "DealAction",
    "PocketOptionClient",
    "SuccessUpdateBalanceEvent",
    "UpdateHistoryFastEvent",
    "UpdateCloseValueItem",
    "UpdateCloseValueListTypeAdapter",
    "constants",
)



import typing

from pocket_option import models
from pocket_option.client import BasePocketOptionClient

if typing.TYPE_CHECKING:
    from pocket_option.types import TypedEventListener

__all__ = ("PocketOptionClient",)


class PocketOptionClientEmit:
    def __init__(self, client: BasePocketOptionClient) -> None:
        self.client = client

    async def ps(self) -> None:
        """No description"""
        await self.client.send("ps")

    async def indicator_load(self) -> None:
        """No description"""
        await self.client.send("indicator/load")

    async def favorite_load(self) -> None:
        """No description"""
        await self.client.send("favorite/load")

    async def price_alert_load(self) -> None:
        """No description"""
        await self.client.send("price-alert/load")

    async def auth(self, data: models.AuthorizationData) -> None:
        """No description


        :type data: models.AuthorizationData
        """
        await self.client.send("auth", data)

    async def subscribe_to_asset(self, asset: models.Asset) -> None:
        """No description


        :type asset: models.Asset
        """
        await self.client.send("subscribeSymbol", asset)

    async def subscribe_for_market_sentiment(self, asset: models.Asset) -> None:
        """No description


        :type asset: models.Asset
        """
        await self.client.send("subfor", asset)

    async def unsubscribe_for_market_sentiment(self, asset: models.Asset) -> None:
        """No description


        :type asset: models.Asset
        """
        await self.client.send("unsubfor", asset)

    async def change_asset(self, data: models.ChangeAssetRequest) -> None:
        """No description


        :type data: models.ChangeAssetRequest
        """
        await self.client.send("changeSymbol", data)

    async def open_deal(self, data: models.OpenDealRequest) -> None:
        """No description


        :type data: models.OpenDealRequest
        """
        await self.client.send("openOrder", data)

    async def copy_signal(self, data: models.CopySignalRequest) -> None:
        """No description


        :type data: models.CopySignalRequest
        """
        await self.client.send("copySignalOrder", data)


class PocketOptionClientOn:
    def __init__(self, client: BasePocketOptionClient) -> None:
        self.client = client

    @typing.overload
    def update_balance(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[models.SuccessUpdateBalanceEvent]], None]": ...

    @typing.overload
    def update_balance(
        self,
        handler: "TypedEventListener[models.SuccessUpdateBalanceEvent]",
    ) -> None: ...

    def update_balance(
        self,
        handler: "TypedEventListener[models.SuccessUpdateBalanceEvent] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[models.SuccessUpdateBalanceEvent]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[models.SuccessUpdateBalanceEvent] | None
        """
        return self.client.add_on("successupdateBalance", handler=handler, model=models.SuccessUpdateBalanceEvent)

    @typing.overload
    def update_history_new_fast(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[models.UpdateHistoryFastEvent]], None]": ...

    @typing.overload
    def update_history_new_fast(
        self,
        handler: "TypedEventListener[models.UpdateHistoryFastEvent]",
    ) -> None: ...

    def update_history_new_fast(
        self,
        handler: "TypedEventListener[models.UpdateHistoryFastEvent] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[models.UpdateHistoryFastEvent]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[models.UpdateHistoryFastEvent] | None
        """
        return self.client.add_on("updateHistoryNewFast", handler=handler, model=models.UpdateHistoryFastEvent)

    @typing.overload
    def update_close_value(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[list[models.UpdateCloseValueItem]]], None]": ...

    @typing.overload
    def update_close_value(
        self,
        handler: "TypedEventListener[list[models.UpdateCloseValueItem]]",
    ) -> None: ...

    def update_close_value(
        self,
        handler: "TypedEventListener[list[models.UpdateCloseValueItem]] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[list[models.UpdateCloseValueItem]]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[list[models.UpdateCloseValueItem]] | None
        """
        return self.client.add_on("updateStream", handler=handler, model=models.UpdateCloseValueListTypeAdapter)

    @typing.overload
    def update_opened_deals(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[list[models.Deal]]], None]": ...

    @typing.overload
    def update_opened_deals(
        self,
        handler: "TypedEventListener[list[models.Deal]]",
    ) -> None: ...

    def update_opened_deals(
        self,
        handler: "TypedEventListener[list[models.Deal]] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[list[models.Deal]]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[list[models.Deal]] | None
        """
        return self.client.add_on("updateOpenedDeals", handler=handler, model=models.DealListTypeAdapter)

    @typing.overload
    def success_open_deal(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[models.Deal]], None]": ...

    @typing.overload
    def success_open_deal(
        self,
        handler: "TypedEventListener[models.Deal]",
    ) -> None: ...

    def success_open_deal(
        self,
        handler: "TypedEventListener[models.Deal] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[models.Deal]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[models.Deal] | None
        """
        return self.client.add_on("successopenOrder", handler=handler, model=models.Deal)

    @typing.overload
    def update_closed_deals(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[list[models.Deal]]], None]": ...

    @typing.overload
    def update_closed_deals(
        self,
        handler: "TypedEventListener[list[models.Deal]]",
    ) -> None: ...

    def update_closed_deals(
        self,
        handler: "TypedEventListener[list[models.Deal]] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[list[models.Deal]]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[list[models.Deal]] | None
        """
        return self.client.add_on("updateClosedDeals", handler=handler, model=models.DealListTypeAdapter)

    @typing.overload
    def update_assets(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[list[models.UpdateAssetItem]]], None]": ...

    @typing.overload
    def update_assets(
        self,
        handler: "TypedEventListener[list[models.UpdateAssetItem]]",
    ) -> None: ...

    def update_assets(
        self,
        handler: "TypedEventListener[list[models.UpdateAssetItem]] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[list[models.UpdateAssetItem]]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[list[models.UpdateAssetItem]] | None
        """
        return self.client.add_on("updateAssets", handler=handler, model=models.UpdateAssetItemListTypeAdapter)

    @typing.overload
    def success_close_deal(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[models.SuccessCloseDealEvent]], None]": ...

    @typing.overload
    def success_close_deal(
        self,
        handler: "TypedEventListener[models.SuccessCloseDealEvent]",
    ) -> None: ...

    def success_close_deal(
        self,
        handler: "TypedEventListener[models.SuccessCloseDealEvent] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[models.SuccessCloseDealEvent]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[models.SuccessCloseDealEvent] | None
        """
        return self.client.add_on("successcloseOrder", handler=handler, model=models.SuccessCloseDealEvent)

    @typing.overload
    def connect(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[None]], None]": ...

    @typing.overload
    def connect(
        self,
        handler: "TypedEventListener[None]",
    ) -> None: ...

    def connect(
        self,
        handler: "TypedEventListener[None] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[None]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[None] | None
        """
        return self.client.add_on("connect", handler=handler, model=None)

    @typing.overload
    def disconnect(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[None]], None]": ...

    @typing.overload
    def disconnect(
        self,
        handler: "TypedEventListener[None]",
    ) -> None: ...

    def disconnect(
        self,
        handler: "TypedEventListener[None] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[None]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[None] | None
        """
        return self.client.add_on("disconnect", handler=handler, model=None)

    @typing.overload
    def success_auth(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[models.SuccessAuthEvent]], None]": ...

    @typing.overload
    def success_auth(
        self,
        handler: "TypedEventListener[models.SuccessAuthEvent]",
    ) -> None: ...

    def success_auth(
        self,
        handler: "TypedEventListener[models.SuccessAuthEvent] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[models.SuccessAuthEvent]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[models.SuccessAuthEvent] | None
        """
        return self.client.add_on("successauth", handler=handler, model=models.SuccessAuthEvent)

    @typing.overload
    def change_market_sentiment(
        self,
        handler: None = None,
    ) -> "typing.Callable[[TypedEventListener[list[models.MarketSentimentItem]]], None]": ...

    @typing.overload
    def change_market_sentiment(
        self,
        handler: "TypedEventListener[list[models.MarketSentimentItem]]",
    ) -> None: ...

    def change_market_sentiment(
        self,
        handler: "TypedEventListener[list[models.MarketSentimentItem]] | None" = None,
    ) -> "None | typing.Callable[[TypedEventListener[list[models.MarketSentimentItem]]], None]":
        """No description

        :param handler: Callback
        :type handler: TypedEventListener[list[models.MarketSentimentItem]] | None
        """
        return self.client.add_on("chafor", handler=handler, model=models.MarketSentimentItemListTypeAdapter)


class PocketOptionClient(BasePocketOptionClient):
    @property
    def on(self) -> PocketOptionClientOn:
        return PocketOptionClientOn(self)

    @property
    def emit(self) -> PocketOptionClientEmit:
        return PocketOptionClientEmit(self)



import contextlib
import typing

from pocket_option.middleware import Middleware
from pocket_option.utils import fix_timestamp, get_json_function

if typing.TYPE_CHECKING:
    from pocket_option.types import JsonValue

__all__ = (
    "FixTypesOnMiddleware",
    "MakeJsonOnMiddleware",
)


UPDATE_ITEMS_NAMES = [
    "id",  # ID актива
    "symbol",  # Символ (#AAPL)
    "label",  # Название (Apple)
    "type",  # Тип (stock, forex, crypto и т.д.)
    "precision",  # Кол-во знаков после запятой
    "payout",  # Выплата (%)
    "min_duration",  # Мин. длительность сделки
    "max_duration",  # Макс. длительность сделки
    "step_duration",  # Шаг длительности
    "volatility_index",  # Индекс волатильности / флаг
    "spread",  # Спред / коэффициент
    "leverage",  # Плечо
    "extra_data",  # Доп. данные (список или null)
    "expire_time",  # Метка времени окончания (timestamp)
    "is_active",  # Активен ли актив
    "timeframes",  # Список доступных таймфреймов [{time: 60}, ...]
    "start_time",  # Время старта (timestamp)
    "default_timeframe",  # Таймфрейм по умолчанию
    "status_code",  # Статус / код состояния
]


class MakeJsonOnMiddleware(Middleware):
    def __init__(self) -> None:
        self.json = get_json_function()

    async def on(self, event: str, data: "str | bytes | JsonValue | None") -> "JsonValue | None":  # noqa: ARG002
        if isinstance(data, str | bytes):
            with contextlib.suppress(Exception):
                return self.json.loads(data)
        return typing.cast("JsonValue", data)


class FixTypesOnMiddleware(Middleware):
    async def on(self, event: str, data: "JsonValue | None") -> "JsonValue | None":  # type: ignore
        if data is None:
            return None
        if event == "updateStream":
            return [
                {
                    "asset": it[0],
                    "timestamp": fix_timestamp(it[1]),
                    "value": it[2],
                }
                for it in typing.cast("list[tuple[str, float, float]]", data)
            ]
        if event == "updateAssets":
            return [dict(zip(UPDATE_ITEMS_NAMES, it, strict=True)) for it in typing.cast("list[list]", data)]
        if event == "chafor":
            return [dict(zip(["asset", "value"], it, strict=True)) for it in typing.cast("list[list]", data)]

        return data



import typing

__all__ = ("DealError", "DealErrorCode")

type DealErrorCode = typing.Literal[
    "min_amount",
    "max_amount",
    "min_duration",
    "max_duration",
    "max_orders",
    "timeout",
    "not_found",
]


class DealError(ValueError):
    def __init__(self, code: DealErrorCode, message: str, extras: dict | None = None) -> None:
        super().__init__(message)
        self.code = code
        self.message = message
        self.extras = extras

    def __str__(self) -> str:
        return f"[{self.code}] {self.message} {self.extras!r}"
